﻿using Microsoft.AspNetCore.Components;
using WF_WebAdmin.Model;
using WF_WebAdmin.Service;

namespace WF_WebAdmin.Pages
{
    /// <summary>
    /// Page permettant de modifier une citation existante.
    /// </summary>
    public partial class Edit
    {
        /// <summary>
        /// Identifiant de la citation à éditer passé en paramètre de la route.
        /// </summary>
        [Parameter]
        public int Id { get; set; }

        /// <summary>
        /// Service pour gérer les citations.
        /// </summary>
        [Inject]
        private IQuoteService quoteService { get; set; }

        /// <summary>
        /// Service pour la navigation entre les pages.
        /// </summary>
        [Inject]
        public NavigationManager NavigationManager { get; set; }

        /// <summary>
        /// Citation actuelle à éditer.
        /// </summary>
        private Quote q { get; set; }

        /// <summary>
        /// Modèle de données pour l'édition de la citation.
        /// </summary>
        private QuoteModel quoteModel = new();

        /// <summary>
        /// Liste des personnages disponibles pour l'édition.
        /// </summary>
        private List<Character> charac = new List<Character>();

        /// <summary>
        /// Liste des sources disponibles pour l'édition.
        /// </summary>
        private List<Source> src = new List<Source>();

        /// <summary>
        /// Méthode appelée lors de l'initialisation de la page.
        /// Elle charge la citation à éditer ainsi que les données associées (personnages, sources).
        /// </summary>
        protected override async Task OnInitializedAsync()
        {
            // Récupère la citation à éditer à partir de l'ID
            q = await quoteService.getOnequote(Id);

            // Initialise le modèle de données avec les informations de la citation
            quoteModel.Content = q.Content;
            quoteModel.Langue = q.Langue;
            quoteModel.Charac = q.Charac;
            quoteModel.TitleSrc = q.TitleSrc;
            quoteModel.Id = q.Id;
            quoteModel.Like = q.Like;
            quoteModel.ImgPath = q.ImgPath;
            quoteModel.DateSrc = q.DateSrc;
            quoteModel.UserProposition = q.UserProposition;
            quoteModel.IsValid = q.IsValid;

            // Charge la liste des personnages et des sources
            charac = await quoteService.getChar();
            src = await quoteService.getSrc();
        }

        /// <summary>
        /// Méthode appelée lors de la soumission valide du formulaire d'édition.
        /// Elle met à jour la citation avec les nouvelles données.
        /// </summary>
        protected async void HandleValidSubmit()
        {
            // Met à jour les informations de la citation
            q.Content = quoteModel.Content;
            q.Langue = quoteModel.Langue;
            q.TitleSrc = quoteModel.TitleSrc;
            q.Charac = quoteModel.Charac;

            // Appelle le service pour mettre à jour la citation dans la base de données
            await quoteService.updateQuote(q);

            // Redirige vers la page des citations à modifier
            NavigationManager.NavigateTo("modifquote");
        }

        /// <summary>
        /// Méthode appelée lors du changement de langue dans le formulaire.
        /// </summary>
        /// <param name="item">Langue sélectionnée.</param>
        /// <param name="checkedValue">Valeur de l'élément vérifié.</param>
        private void OnlangChange(string item, object checkedValue)
        {
            // Si la langue est "fr" ou "en", met à jour la langue dans le modèle de données
            if (item == "fr" || item == "en")
            {
                quoteModel.Langue = item;
            }    
        }
    }
}
