using Blazorise.DataGrid;
using Microsoft.AspNetCore.Components;
using WF_WebAdmin.Model;
using WF_WebAdmin.Service;

namespace WF_WebAdmin.Pages
{
    /// <summary>
    /// Composant Blazor pour la gestion des quiz, permettant leur modification et suppression.
    /// </summary>
    public partial class ModifQuiz
    {
        /// <summary>
        /// Liste des quiz à afficher dans le tableau.
        /// </summary>
        private Quiz[] quiz;

        /// <summary>
        /// Valeur maximale d'éléments par page.
        /// </summary>
        private int MaxValue = 5;

        /// <summary>
        /// Nombre total d'éléments dans la liste des quiz.
        /// </summary>
        private int totalItem;

        /// <summary>
        /// Indicateur de l'affichage de la popup de modification du quiz.
        /// </summary>
        private bool showEditQuiz = false;

        /// <summary>
        /// Quiz actuellement sélectionné pour modification.
        /// </summary>
        private Quiz? selectedQuiz;

        /// <summary>
        /// Indicateur de l'affichage de la popup de suppression.
        /// </summary>
        private bool showPopupDelete = false;

        /// <summary>
        /// Numéro de la page actuellement affichée.
        /// </summary>
        private int page = 1;

        /// <summary>
        /// Service pour gérer les quiz injecté.
        /// </summary>
        [Inject]
        public IQuizService QuizService { get; set; }

        /// <summary>
        /// Méthode appelée pour récupérer les données de la page actuelle dans le tableau.
        /// </summary>
        /// <param name="e">Arguments de lecture de données pour le DataGrid.</param>
        private async Task OnReadData(DataGridReadDataEventArgs<Quiz> e)
        {
            if (e.CancellationToken.IsCancellationRequested)
            {
                return;
            }

            var response = await QuizService.getSommeQuiz(e.PageSize, e.Page);

            if (!e.CancellationToken.IsCancellationRequested)
            {
                totalItem = await QuizService.getNbQuiz();
                quiz = response.ToArray();
                page = e.Page;
            }
        }

        /// <summary>
        /// Méthode pour afficher la popup de modification d'un quiz.
        /// </summary>
        /// <param name="quiz">Quiz à modifier.</param>
        private void OnEditButtonClicked(Quiz quiz)
        {
            if (quiz == null) return;
            selectedQuiz = quiz;
            showEditQuiz = true;
        }

        /// <summary>
        /// Méthode pour fermer la popup (modification ou suppression).
        /// </summary>
        private void ClosePopup()
        {
            showEditQuiz = false;
            showPopupDelete = false;
            selectedQuiz = null;
        }

        /// <summary>
        /// Méthode pour modifier un quiz sélectionné.
        /// </summary>
        private async Task EditQuiz()
        {
            if (selectedQuiz != null)
            {
                await QuizService.updateQuiz(selectedQuiz);
                selectedQuiz = null;
                ClosePopup();
            }
        }

        /// <summary>
        /// Méthode pour afficher la popup de suppression d'un quiz.
        /// </summary>
        /// <param name="q">Quiz à supprimer.</param>
        private void OnDelete(Quiz q)
        {
            selectedQuiz = q;
            showPopupDelete = true;
        }

        /// <summary>
        /// Méthode pour supprimer un quiz sélectionné.
        /// </summary>
        private async void RemoveQuote()
        {
            if (selectedQuiz != null)
            {
                await QuizService.removeQuiz(selectedQuiz.Id);
                selectedQuiz = null;
                var response = await QuizService.getSommeQuiz(MaxValue, page);
                quiz = response.ToArray();
            }
            showPopupDelete = false;
        }
    }
}
