using System.Text.Json;
using WF_WebAdmin.Model;

namespace WF_WebAdmin.Service;

/// <summary>
/// Service de gestion des utilisateurs utilisant un fichier JSON comme stockage.
/// </summary>
public class UserServiceStub : IUserService
{
    private readonly string _jsonFilePath = Path.Combine(Environment.CurrentDirectory, "wwwroot", "fake_data_users.json");

    /// <summary>
    /// Sauvegarde la liste des utilisateurs dans le fichier JSON.
    /// </summary>
    /// <param name="users">Liste des utilisateurs à sauvegarder.</param>
    public async Task saveUsersJson(List<User> users)
    {
        var json = JsonSerializer.Serialize(users, new JsonSerializerOptions { WriteIndented = true });
        await File.WriteAllTextAsync(_jsonFilePath, json);
    }

    /// <summary>
    /// Supprime un utilisateur de la liste et met à jour le fichier JSON.
    /// </summary>
    /// <param name="user">Utilisateur à supprimer.</param>
    public async Task removeUser(User user)
    {
        var data = await getAllUser();
        var u = data.FirstOrDefault(p => p.Id == user.Id);
        if (u != null)
        {
            data.Remove(u);
            await saveUsersJson(data);
        }
    }

    /// <summary>
    /// Met à jour le rôle d'un utilisateur pour le passer en administrateur.
    /// </summary>
    /// <param name="user">Utilisateur à promouvoir.</param>
    public Task updateRole(User user)
    {
        user.IsAdmin = true;
        return updateUser(user);
    }

    /// <summary>
    /// Rétrograde un administrateur en utilisateur normal.
    /// </summary>
    /// <param name="user">Utilisateur à rétrograder.</param>
    public Task downgradeRole(User user)
    {
        user.IsAdmin = false;
        return updateUser(user);
    }

    /// <summary>
    /// Récupère tous les utilisateurs stockés dans le fichier JSON.
    /// </summary>
    /// <returns>Une liste d'utilisateurs.</returns>
    public async Task<List<User>> getAllUser()
    {
        if (!File.Exists(_jsonFilePath))
        {
            Console.Out.WriteLine($"{_jsonFilePath} not found");
            return new List<User>();
        }

        var json = await File.ReadAllTextAsync(_jsonFilePath);
        return JsonSerializer.Deserialize<List<User>>(json) ?? new List<User>();
    }

    /// <summary>
    /// Récupère une liste paginée d'utilisateurs.
    /// </summary>
    /// <param name="nb">Nombre d'utilisateurs par page.</param>
    /// <param name="page">Numéro de la page.</param>
    /// <returns>Une liste d'utilisateurs correspondant à la page demandée.</returns>
    public async Task<List<User>> getSomeUser(int nb, int page)
    {
        var users = await getAllUser();
        if ((page - 1) * nb + nb > users.Count())
        {
            return users.GetRange(users.Count() - nb, nb);
        }
        return users.GetRange((page - 1) * nb, nb);
    }

    /// <summary>
    /// Récupère un utilisateur spécifique en fonction de son ID.
    /// </summary>
    /// <param name="id">ID de l'utilisateur recherché.</param>
    /// <returns>L'utilisateur correspondant à l'ID ou null s'il n'existe pas.</returns>
    public async Task<User> getOneUser(int id)
    {
        var data = await getAllUser();
        var u = data.FirstOrDefault(p => p.Id == id);
        return u;
    }

    /// <summary>
    /// Recherche des utilisateurs selon des critères spécifiques.
    /// </summary>
    /// <param name="reserch">Terme de recherche.</param>
    /// <param name="args">Liste des arguments supplémentaires pour affiner la recherche.</param>
    /// <returns>Une liste d'utilisateurs correspondant aux critères de recherche.</returns>
    public Task<List<User>> reserchUsers(string reserch, List<string> args)
    {
        throw new NotImplementedException();
    }

    /// <summary>
    /// Récupère le nombre total d'utilisateurs enregistrés.
    /// </summary>
    /// <returns>Nombre total d'utilisateurs.</returns>
    public async Task<int> getNbUser()
    {
        var data = await getAllUser();
        return data.Count;
    }

    /// <summary>
    /// Met à jour les informations d'un utilisateur existant.
    /// </summary>
    /// <param name="user">Utilisateur avec les nouvelles informations.</param>
    public async Task updateUser(User user)
    {
        var data = await getAllUser();
        var person = data.FirstOrDefault(p => p.Id == user.Id);
        if (person != null)
        {
            person.Name = user.Name;
            person.Email = user.Email;
            person.Image = user.Image;
            person.IsAdmin = user.IsAdmin;
            await saveUsersJson(data);
        }
    }
}
